@@ -1,35 +1,41 @@
 # coding=utf-8
+# Copyright 2018 Sascha Schirra
 #
-# Copyright 2014 Sascha Schirra
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# This file is part of Ropper.
+# 1. Redistributions of source code must retain the above copyright notice, this
+# list of conditions and the following disclaimer.
 #
-# Ropper is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution.
 #
-# Ropper is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# 3. Neither the name of the copyright holder nor the names of its contributors
+# may be used to endorse or promote products derived from this software without
+# specific prior written permission.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" A ND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 from ropper.common.abstract import *
 from ropper.common.enum import Enum
 from ropper.common.error import NotSupportedError
-from ropper.search import Searcher
-from ropper.search import Searcherx86
-from ropper.search import SearcherARM
+from ropper.search import Searcher, Searcherx86, SearcherARM, SearcherMIPS
 from re import compile
 from capstone import *
 from . import gadget
 try:
     import sys
-    if sys.version_info.major < 3:
-        import archinfo
+    import archinfo
 except:
     pass
 
@@ -68,7 +74,7 @@ class Architecture(AbstractSingleton):
         self._initGadgets()
         self._initBadInstructions()
         self._initCategories()
-        
+
         self._initEndianess(endianess)
 
         self._endings[gadget.GadgetType.ALL] = self._endings[
@@ -91,12 +97,12 @@ class Architecture(AbstractSingleton):
                 tmp = []
                 for pattern, size in self.endings[key]:
                     tmp.append((pattern[::-1], size))
-                self.endings[key] = tmp 
+                self.endings[key] = tmp
 
     @property
     def info(self):
         return self._info
-    
+
 
     @property
     def ksarch(self):
@@ -170,7 +176,7 @@ class ArchitectureX86(Architecture):
                         b'\x83\xc4\x04[\x58-\x5f]\xc3', # add esp, 4; pop reg; ret
                         b'[\x58-\x5f]\x83\xc4\x04\xc3', # pop reg; add esp, 4; ret
                         b'\x83\xc4\x08\xc3',            # add esp, 8; ret;
-                        b'\xff\x54\x24[\x08\x14\x1c\x2c\x44\x50]',            # call [esp+n] 
+                        b'\xff\x54\x24[\x08\x14\x1c\x2c\x44\x50]',            # call [esp+n]
                         b'\xff\x64\x24[\x08\x14\x1c\x2c\x44\x50]',            # jmp [esp+n]
                         b'\xff\x65[\x0c\x24\x30\xfc\xf4\xe8]',                            # jmp [ebp+n]
                         b'\xff\x55[\x0c\x24\x30\xfc\xf4\xe8]'                             # call [ebp+n]
@@ -292,6 +298,8 @@ class ArchitectureMips(Architecture):
         if 'archinfo' in globals():
             self._info = archinfo.ArchMIPS32()
 
+        self._searcher = SearcherMIPS()
+
     def _initGadgets(self):
         super(ArchitectureMips, self)._initGadgets()
         self._endings[gadget.GadgetType.ROP] = []
@@ -306,6 +314,8 @@ class ArchitectureMipsBE(ArchitectureMips):
         super(ArchitectureMipsBE, self).__init__(Endianess.BIG)
         self._name = 'MIPSBE'
         self._mode |= CS_MODE_BIG_ENDIAN
+        if 'keystone' in globals():
+            self._ksarch = (self._ksarch[0], self._ksarch[1] + keystone.KS_MODE_BIG_ENDIAN)
 
 class ArchitectureMips64(ArchitectureMips):
 
@@ -332,6 +342,8 @@ class ArchitectureMips64BE(ArchitectureMips64):
         super(ArchitectureMips64BE, self).__init__(Endianess.BIG)
         self._name = 'MIPS64BE'
         self._mode |= CS_MODE_BIG_ENDIAN
+        if 'keystone' in globals():
+            self._ksarch = (self._ksarch[0], self._ksarch[1] + keystone.KS_MODE_BIG_ENDIAN)
 
 class ArchitectureArm(Architecture):
 
@@ -350,23 +362,25 @@ class ArchitectureArm(Architecture):
         self._endings[gadget.GadgetType.ROP] = [(b'[\x01-\xff]\x80\xbd\xe8', 4)] # pop {[reg]*,pc}
         self._endings[gadget.GadgetType.JOP] = [(b'[\x10-\x1e]\xff\x2f\xe1', 4), # bx <reg>
                                                 (b'[\x30-\x3e]\xff\x2f\xe1', 4), # blx <reg>
-
+                                                (b'[\x00-\x0f]\xf0\xa0\xe1', 4), # mov pc, <reg>
                                                 (b'\x01\x80\xbd\xe8', 4)] # ldm sp! ,{pc}
 
 
 class ArchitectureArmBE(ArchitectureArm):
-    
+
     def __init__(self):
         super(ArchitectureArmBE, self).__init__(Endianess.BIG)
         self._name = 'ARMBE'
         self._mode |= CS_MODE_BIG_ENDIAN
+        if 'keystone' in globals():
+            self._ksarch = (self._ksarch[0], self._ksarch[1] + keystone.KS_MODE_BIG_ENDIAN)
 
     def _initEndianess(self, endianess):
-        super(ArchitectureArmBE, self)._initEndianess(endianess)    
+        super(ArchitectureArmBE, self)._initEndianess(endianess)
         self._endings[gadget.GadgetType.ROP] = [(b'\xe8\xbd\x80[\x01-\xff]', 4)] # pop {[reg]*,pc}
         self._endings[gadget.GadgetType.JOP] = [(b'\xe1\x2f\xff[\x10-\x1e]', 4), # bx <reg>
                                                 (b'\xe1\x2f\xff[\x30-\x3e]', 4), # blx <reg>
-
+                                                (b'\xe1\xa0\xf0[\x00-\x0f]', 4), # mov pc, <reg>
                                                 (b'\xe8\xdb\x80\x01', 4)] # ldm sp! ,{pc}
 
 class ArchitectureArmThumb(Architecture):
@@ -402,7 +416,7 @@ class ArchitectureArm64(Architecture):
         if 'archinfo' in globals():
             self._info = archinfo.ArchAArch64()
         if 'keystone' in globals():
-            self._ksarch = (keystone.KS_ARCH_ARM64, keystone.KS_MODE_BIG_ENDIAN)
+            self._ksarch = (keystone.KS_ARCH_ARM64, keystone.KS_MODE_LITTLE_ENDIAN)
 
     def _initGadgets(self):
         super(ArchitectureArm64, self)._initGadgets()
@@ -412,8 +426,8 @@ class ArchitectureArm64(Architecture):
 
         self._endings[gadget.GadgetType.JOP] = [(b'[\x00\x20\x40\x60\x80\xa0\xc0\xe0][\x00-\x02]\x1f\xd6', 4), # br <reg>
                                                 (b'[\x00\x20\x40\x60\x80]\x03\x1f\xd6', 4), # br <reg>
-                                                (b'[\x00\x20\x40\x60\x80\xa0\xc0\xe0][\x00-\x02]\x3f\xd6', 4), # blr <reg>
-                                                (b'[\x00\x20\x40\x60\x80]\x03\x3f\xd6', 4)] # blr <reg>
+                                                (b'[\x00\x20\x40\x60\x80\xa0\xc0\xe0][\x00-\x02]\\?\xd6', 4), # blr <reg>
+                                                (b'[\x00\x20\x40\x60\x80]\x03\\?\xd6', 4)] # blr <reg>
 
 
 
